/*
 * Author:          Patrick-Christopher Mattulat
 * Company:         Lynar Studios
 * E-Mail:          webmaster@lynarstudios.com
 * Created:         2020-08-14
 * Changed:         2024-05-31
 *
 * */

#ifndef LS_STD_DOUBLE_HPP
#define LS_STD_DOUBLE_HPP

#include <ls-std/core/Class.hpp>
#include <ls-std/core/definition.hpp>
#include <ls-std/core/interface/IBoxing.hpp>

namespace ls::std::boxing
{
  ls_std_class Double final : public core::Class, public core::interface_type::IBoxing
  {
    public:

      Double();
      explicit Double(double _value);
      ~Double() noexcept override;

      // assignment operators

      Double &operator=(double _value);

      // arithmetic operators

      double operator-() const;
      double operator+(const Double &_double) const;
      double operator+(double _value) const;
      double operator*(const Double &_double) const;
      double operator*(double _value) const;
      double operator-(const Double &_double) const;
      double operator-(double _value) const;
      double operator/(const Double &_double) const;
      double operator/(double _value) const;

      // compound operators

      Double &operator+=(const Double &_double);
      Double &operator+=(double _value);
      Double &operator-=(const Double &_double);
      Double &operator-=(double _value);
      Double &operator*=(const Double &_double);
      Double &operator*=(double _value);
      Double &operator/=(const Double &_double);
      Double &operator/=(double _value);

      // comparison operators

      bool operator==(const Double &_double) const;
      bool operator==(double _value) const;
      bool operator!=(const Double &_double) const;
      bool operator!=(double _value) const;
      bool operator>(const Double &_double) const;
      bool operator>(double _value) const;
      bool operator>=(const Double &_double) const;
      bool operator>=(double _value) const;
      bool operator<(const Double &_double) const;
      bool operator<(double _value) const;
      bool operator<=(const Double &_double) const;
      bool operator<=(double _value) const;

      // increment / decrement operator

      void operator++();
      void operator--();

      // implementation

      void parse(const ::std::string &_parseText) override;
      [[nodiscard]] ::std::string toString() override;

      // additional functionality

      [[nodiscard]] double getEpsilon() const;
      [[nodiscard]] double getValue() const;
      void setEpsilon(double _epsilon);

    private:

      double epsilon{};
      double value{};

      void _assignEpsilon(double _epsilon);
      [[nodiscard]] static ::std::string _fetchClassName();
  };
}

#endif
