/*
 * Author:          Patrick-Christopher Mattulat
 * Company:         Lynar Studios
 * E-Mail:          webmaster@lynarstudios.com
 * Created:         2020-08-07
 * Changed:         2024-05-31
 *
 * */

#ifndef LS_STD_INTEGER_HPP
#define LS_STD_INTEGER_HPP

#include <ls-std/core/Class.hpp>
#include <ls-std/core/definition.hpp>
#include <ls-std/core/interface/IBoxing.hpp>

namespace ls::std::boxing
{
  ls_std_class Integer final : public core::Class, public core::interface_type::IBoxing
  {
    public:

      explicit Integer(int _value);
      Integer();
      ~Integer() noexcept override;

      // assignment operators

      Integer &operator=(int _value);

      // arithmetic operators

      int operator-() const;
      int operator+(const Integer &_integer) const;
      int operator+(int _value) const;
      int operator*(const Integer &_integer) const;
      int operator*(int _value) const;
      int operator-(const Integer &_integer) const;
      int operator-(int _value) const;
      int operator/(const Integer &_integer) const;
      int operator/(int _value) const;
      int operator%(const Integer &_integer) const;
      int operator%(int _value) const;

      // compound operators

      Integer &operator+=(const Integer &_integer);
      Integer &operator+=(int _value);
      Integer &operator-=(const Integer &_integer);
      Integer &operator-=(int _value);
      Integer &operator*=(const Integer &_integer);
      Integer &operator*=(int _value);
      Integer &operator/=(const Integer &_integer);
      Integer &operator/=(int _value);

      // comparison operators

      bool operator==(const Integer &_integer) const;
      bool operator==(int _value) const;
      bool operator!=(const Integer &_integer) const;
      bool operator!=(int _value) const;
      bool operator>(const Integer &_integer) const;
      bool operator>(int _value) const;
      bool operator>=(const Integer &_integer) const;
      bool operator>=(int _value) const;
      bool operator<(const Integer &_integer) const;
      bool operator<(int _value) const;
      bool operator<=(const Integer &_integer) const;
      bool operator<=(int _value) const;

      // logical operators

      friend bool operator!(const Integer &_integer)
      {
        return !_integer.value;
      }

      // increment / decrement operator

      void operator++();
      void operator--();

      // implementation

      void parse(const ::std::string &_parseText) override;
      [[nodiscard]] ::std::string toString() override;

      // additional functionality

      [[nodiscard]] int getValue() const;

    private:

      int value{};

      [[nodiscard]] static ::std::string _fetchClassName();
  };
}

#endif
